#!/usr/bin/env python
# -*- coding: utf-8 -*-
###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.editor module of the BEAT platform.           #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

"""
The main entry for beat.editor (click-based) scripts.
"""

import os
import sys

import click
import pkg_resources

from click_plugins import with_plugins
from PyQt5.QtWidgets import QApplication

from beat.cmdline.click_helper import AliasedGroup
from beat.cmdline.config import Configuration
from beat.cmdline.decorators import raise_on_error
from beat.cmdline.decorators import verbosity_option

from ..utils import setup_logger
from ..widgets.mainwindow import MainWindow

EPILOG = """\b
Example:
        $ beat editor start

        """


@with_plugins(pkg_resources.iter_entry_points("beat.editor.cli"))
@click.group(cls=AliasedGroup)
@click.pass_context
def editor(ctx):
    """beat.editor commands."""
    pass


@editor.command(epilog=EPILOG)
@click.option(
    "--prefix",
    "-p",
    help="Overrides the prefix of your local data. If not set use the value from your RC file [default: %(prefix)s]",
    type=click.STRING,
)
@click.option(
    "--cache",
    "-c",
    help="Overrides the cache prefix. If not set, use the value from your RC file, otherwise defaults to `<prefix>/%(cache)s'",
    type=click.STRING,
)
@verbosity_option()
@click.pass_context
@raise_on_error
def start(ctx, prefix, cache):
    """Start the beat editor
    """

    ctx.meta["prefix"] = prefix
    ctx.meta["cache"] = cache

    completions = dict(
        prog=os.path.basename(sys.argv[0]),
        version=pkg_resources.require("beat.editor")[0].version,
    )

    completions.update(Configuration({}).as_dict())

    # Check that we are in a BEAT working folder
    logger = setup_logger("beat.editor", ctx.meta["verbosity"])

    config = Configuration(ctx.meta)
    logger.info("BEAT prefix set to `%s'", config.path)
    logger.info("BEAT cache set to `%s'", config.cache)

    app = QApplication(sys.argv)
    mainwindow = MainWindow()
    mainwindow.set_prefix_root(config.path)
    mainwindow.show()
    return app.exec_()
